package net.java.dev.esjug.util.frameworks.crud;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import net.java.dev.esjug.util.persistence.PersistentObject;

import com.opensymphony.xwork2.ActionSupport;
import com.opensymphony.xwork2.Preparable;

/**
 * Base class for Struts action classes that provide controller
 * functionality for CRUD use cases. It integrates with the 
 * CrudService interface, forming a CRUD framework.
 * 
 * @see net.java.dev.esjug.util.frameworks.crud.CrudService
 * @author Vitor Souza (vitorsouza@gmail.com)
 */
public abstract class CrudAction<T extends PersistentObject<I, ? extends Serializable>, I extends Serializable> extends ActionSupport implements Preparable {
	/** Logging. */
	private static final Log logger = LogFactory.getLog(CrudAction.class);;
	
	/** Represents a result that should list the objects. */
	public static final String LISTING = "listing";

	/** Represents a result that should confirm the exclusion of objects. */
	public static final String CONFIRMATION = "confirmation";
	
	/** The model in a model-driven approach. It holds information of the objects that is to be created/updated. */
	protected T model;

	/** Input parameter: various object identifiers. */
	protected I[] ids;

	/** Output objects: collection of objects that are retrieved from the business layer. */
	protected Collection<T> objects;

	/**
	 * @return Returns the objects.
	 */
	public Collection<T> getObjects() {
		return objects;
	}

	/**
	 * @param ids The ids to set.
	 */
	public void setIds(I[] ids) {
		this.ids = ids;
	}

	/**
	 * @return Returns the model.
	 */
	public T getModel() {
		return model;
	}
	
	/* (non-Javadoc)
	 * @see com.opensymphony.xwork.Preparable#prepare()
	 */
	public void prepare() throws Exception {
		logger.warn("Preparing...");

		// Creates the model object.
		model = createModel();
	}

	/**
	 * Obtains the service class that is responsible for executing the scenarios of the CRUD use case.
	 * This method should be implemented by the concrete subclasses.
	 * 
	 * @return The CRUD service object.
	 */
	protected abstract CrudService<T, I> getAplCrud();
	
	/**
	 * Creates a new model object. This method should be implemented by the concrete subclasses, since
	 * the CrudAction class doesn't know to which classes it will be applied.
	 * 
	 * @return Newly created model object.
	 */
	protected abstract T createModel();

	/**
	 * Default action. Lists all existing objects.
	 * 
	 * @return The result 'listing', which indicates that the objects should be listed.
	 */
	public String execute() throws Exception {
		logger.warn("Displaying the list...");

		// Retrieves all objects from the business tier and makes them available.
		objects = getAplCrud().list();
		return LISTING;
	}
	
	/**
	 * Input action. Displays the form for an object creation or update. Should be used with the
	 * actionName!input.action sintax.
	 * 
	 * @return The result 'input', which indicates that the form should be displayed.
	 */
	public String form() throws Exception {
		logger.warn("Displaying the form... model = " + model);

		// If an id has been specified, load an object for update.
		if ((model != null) && (model.getId() != null)) {
			T object = getAplCrud().retrieve(model.getId()); 
			if (object != null) model = object;
		}
		return INPUT;
	}

	/**
	 * Confirmation action. Asks for confirmation on the deletion of some objects.
	 * 
	 * @return The result 'confirmation', which indicates that deletion confirmation should be asked.
	 */
	public String executeConfirmation() throws Exception {
		logger.warn("Displaying confirmation page... ids = " + ids);

		objects = new ArrayList<T>();

		// Checks if a single object id has been informed.
		if (model.getId() != null) {
			T objeto = getAplCrud().retrieve(model.getId()); 
			if (objeto != null) objects.add(objeto);
		}
		
		// Checks if multiple object ids have been informed.
		if ((ids != null) && (ids.length > 0)) {
			for (int i = 0; i < ids.length; i++) {
				T objeto = getAplCrud().retrieve(ids[i]);
				if (objeto != null) objects.add(objeto);
			}
		}
		
		// If no object id was informed, sets an error message and returns to the listing.
		if (objects.size() == 0) {
			addActionError(getText("error.noObjectSelected"));
			return execute();
		}
		
		// Otherwise, proceed to deletion confirmation.
		return CONFIRMATION;
	}

	/**
	 * Save action. Creates a new object or updates an object given the object's data.
	 * 
	 * @return The 'success' result, indicating that all went well.
	 */
	public String executeSave() throws Exception {
		logger.warn("Saving object... model = " + model);

		// Check if it's a creation or update. Checks for errors before proceeding.
		if (model.getId() == null) {
			getAplCrud().validateCreate(model);
			getAplCrud().create(model);
		}
		else {
			getAplCrud().validateUpdate(model.getId(), model);
			getAplCrud().update(model.getId(), model);
		}

		// Proceed with success.
		return SUCCESS;
	}

	/**
	 * Delete action. Deletes an object or a collection of objects.
	 * 
	 * @return The 'success' result, indication that all went well.
	 */
	public String executeDelete() throws Exception {
		logger.warn("Deleting objects... ids = " + ids);

		List<I> idList = new ArrayList<I>();
		
		// Checks if a single object has been selected for deletion.
		if (model.getId() != null) idList.add(model.getId());

		// Checks if multiple objects has been selected for deletion.
		if ((ids != null) && (ids.length > 0)) idList.addAll(Arrays.asList(ids)); 
			
		// If no object id was informed, sets an error message and returns to the listing.
		if (idList.size() == 0) {
			addActionError(getText("error.noObjectSelected"));
			return execute();
		}
		
		// Checks for errors.
		getAplCrud().validateDelete(idList);

		// Proceed with the exclusion.
		for (I id : ids) {
			getAplCrud().delete(id);
		}

		// Proceed with success.
		return SUCCESS;
	}
}
