package net.java.dev.esjug.util.frameworks.crud;

import java.io.Serializable;
import java.util.Collection;

/**
 * Interface for application classes (service classes) that implement
 * CRUD use cases (use cases that define four scenarios: create, 
 * retrieve, update and delete).
 * 
 * </p><p>
 * This interface defines methods that service classes should provide
 * implementation for. It uses two generic types: <code>T</code> and
 * <code>I</code>. The latter represents the object identifier, which
 * is used in the scenarios where the object is informed. The former
 * represents the class that is manipulated by the use case.
 * 
 * </p><p>
 * For instance, to implement a CRUD service for a Product class using
 * Long ids, use:
 * 
 * </p><pre>public class ProductCrudService implements CrudService<Product, Long></pre>
 * 
 * <p>
 * This interface can be used together with interfaces/classes that
 * are used in the presentation layer (GUI, Web, etc.), forming a CRUD
 * framework that makes the task of implementing CRUD use cases easier.
 * It also provides a default implementation, CrudServiceSupport.
 *
 * @see net.java.dev.esjug.util.frameworks.crud.CrudServiceSupport
 * @author Vitor Souza (vitorsouza@gmail.com)
 */
public interface CrudService<T, I> extends Serializable {
	/**
	 * Checks if there are errors with object creation.
	 * 
	 * @param valueObject Object that contains the data of the object that is to be created.
	 * @throws Exception In case business constraints aren't followed.
	 */
	void validateCreate(T valueObject) throws Exception;

	/**
	 * Checks if there are errors with object update.
	 * 
	 * @param id Identifier of the object that is to be updated.
	 * @param valueObject Object that contains the data of the object that is to be updated.
	 * @throws Exception In case business constraints aren't followed.
	 */
	void validateUpdate(I id, T valueObject) throws Exception;

	/**
	 * Checks if there are errors with object deletion.
	 * 
	 * @param id Identifier of the object that is to be deleted.
	 * @throws Exception In case business constraints aren't followed.
	 */
	void validateDelete(I id) throws Exception;

	/**
	 * Checks if there are errors with multiple object deletion. 
	 * 
	 * @param ids Identifiers of the objects that are to be deleted.
	 * @throws Exception In case business constraints aren't followed.
	 */
	void validateDelete(Collection<I> ids) throws Exception;
	
	/**
	 * Creates an object.
	 * 
	 * @param valueObject Object that contains the data of the object that is to be created.
	 */
	void create(T valueObject);

	/**
	 * Retrieves an object.
	 * 
	 * @param id Identifier of the object that is to be retrieved.
	 * @return The retrieved object.
	 */
	T retrieve(I id);

	/**
	 * Updates an object.
	 * 
	 * @param id Identifier of the object that is to be updated.
	 * @param valueObject Object that contains the data of the object that is to be updated.
	 */
	void update(I id, T valueObject);

	/**
	 * Deletes an object.
	 * 
	 * @param id Identifier of the object that is to be deleted.
	 */
	void delete(I id);

	/**
	 * Retrieves all objects of the class.
	 * 
	 * @return Collection of all objects of the class.
	 */
	Collection<T> list();
}
