package net.java.dev.esjug.util.frameworks.crud;

import java.util.Collection;
import java.util.Observable;


/**
 * Abstract class that implements CrudService.
 * 
 * </p><p>
 * This implementation offers notification methods for observers.
 * Subclasses need to implement methods <code>do<i>Scenario</i>()</code>
 * with the business logic specific for each scenario.
 * 
 * @see net.java.dev.esjug.util.frameworks.crud.CrudService
 * @author Vitor Souza (vitorsouza@gmail.com)
 */
public abstract class CrudServiceSupport<T, I> extends Observable implements CrudService<T, I> {
	/**
	 * Notifies that an object has been created.
	 * 
	 * @param object Object that has just been created.
	 */
	protected final void notifyCreate(T object) {
		CrudNotification notification = CrudNotification.CREATE;
		notification.setObject(object);
		setChanged();
		notifyObservers(notification);
	}

	/**
	 * Notifies that an object has been updated.
	 * 
	 * @param object Object that has just been updated.
	 */
	protected final void notifyUpdate(T object) {
		CrudNotification notification = CrudNotification.UPDATE;
		notification.setObject(object);
		setChanged();
		notifyObservers(notification);
	}

	/**
	 * Notifies that an object has been deleted.
	 * 
	 * @param object Object that has just been deleted.
	 */
	protected final void notifyDelete(T object) {
		CrudNotification notification = CrudNotification.DELETE;
		notification.setObject(object);
		setChanged();
		notifyObservers(notification);
	}

	/**
	 * Notifies that an unspecified operation has happened.
	 */
	protected final void notifyOperation() {
		CrudNotification notification = CrudNotification.UNSPECIFIED;
		setChanged();
		notifyObservers(notification);
	}

	/* (non-Javadoc)
	 * @see net.java.dev.esjug.util.frameworks.crud.CrudService#validateUpdate(I, T)
	 */
	public void validateUpdate(I id, T valueObject) throws Exception { }

	/* (non-Javadoc)
	 * @see net.java.dev.esjug.util.frameworks.crud.CrudService#validateCreate(T)
	 */
	public void validateCreate(T valueObject) throws Exception { }

	/* (non-Javadoc)
	 * @see net.java.dev.esjug.util.frameworks.crud.CrudService#validateDelete(java.util.Collection)
	 */
	public void validateDelete(Collection<I> ids) throws Exception { }

	/* (non-Javadoc)
	 * @see net.java.dev.esjug.util.frameworks.crud.CrudService#validateDelete(I)
	 */
	public void validateDelete(I id) throws Exception { }

	/* (non-Javadoc)
	 * @see net.java.dev.esjug.util.frameworks.crud.CrudService#create(T)
	 */
	public void create(T valueObject) {
		T object = doCreate(valueObject);
		notifyCreate(object);
	}

	/* (non-Javadoc)
	 * @see net.java.dev.esjug.util.frameworks.crud.CrudService#retrieve(I)
	 */
	public T retrieve(I id) {
		return doRetrieve(id);
	}

	/* (non-Javadoc)
	 * @see net.java.dev.esjug.util.frameworks.crud.CrudService#update(I, T)
	 */
	public void update(I id, T valueObject) {
		T object = doUpdate(id, valueObject);
		notifyUpdate(object);
	}

	/* (non-Javadoc)
	 * @see net.java.dev.esjug.util.frameworks.crud.CrudService#delete(I)
	 */
	public void delete(I id) {
		T object = doDelete(id);
		notifyDelete(object);
	}

	/**
	 * Business procedures to create an object. Should be implemented by subclasses.
	 * 
	 * @param valueObject Object that contains the data of the object that is to be created.
	 * @return Object that has just been created.
	 */
	protected abstract T doCreate(T valueObject);

	/**
	 * Business procedures to retrieve an object. Should be implemented by subclasses.
	 * 
	 * @param id Identifier of the object that is to be retrieved.
	 * @return Object that has just been retrieved.
	 */
	protected abstract T doRetrieve(I id);

	/**
	 * Business procedures to update an object. Should be implemented by subclasses.
	 * 
	 * @param id Identifier of the object that is to be updated.
	 * @param valueObject Object that contains the data of the object that is to be updated.
	 * @return Object that has just been updated.
	 */
	protected abstract T doUpdate(I id, T valueObject);

	/**
	 * Business procedures to delete an object. Should be implemented by subclasses.
	 * 
	 * @param id Identifier of the object that is to be deleted.
	 * @return Object that has just been deleted.
	 */
	protected abstract T doDelete(I id);
}
